package org.glitched.engine.graphic;

import org.glitched.util.Color;
import org.glitched.util.Vector2f;
import org.newdawn.slick.Image;

/**
 * Sprite class : est rendu par la classe Renderer
 *
 * @author Sparamoule
 */
public class Sprite {
    
    
    private Image image;
    
    /**
     * Opacité du sprite : 1 = Opaque (sauf texture) 0 = invisible
     */
    private float opacity = 1.f;
    
    
    /**
     * Position du sprite
     */
    private Vector2f position = new Vector2f(0,0);
    
    
    /**
     * Taille du sprite
     */
    private Vector2f size = new Vector2f(0,0);
    
    /**
     * Distance par rapport à la vue <br/>
     * <ul>
     *      <li> sous ou egal à 0 - fixe, en premier plan</li>
     *      <li> au-dessus de 0 - dépendant de la vue, influencé par la pseudo-perspective</li>
     *      <li> 1 - normal </li>
     * </ul>       
     * @since 1.0
     */
    private float depth = 1;
    
    /**
     * Couche de rendu
     */
    private int layer = 0;
    
    /**
     * Retournement de la texture sur l'axe X -gauche/droite
     */
    
    private boolean flipX = false;
    
    /**
     * Retournement de la texture sur l'axe Y -haut/bas
     */
    private boolean flipY = false;
    
    /**
     * Couleur globale du sprite (multiplié par la texture)
     */
    private Color color = new Color(1,1,1);
    
    /**
     * Constructeur par copie
     * @param s Sprite à copier
     */
    public Sprite(Sprite s) {
        this.image = s.image;
        this.color = s.color;
        this.depth = s.depth;
        this.flipX = s.flipX;
        this.flipY = s.flipY;
        this.layer = s.layer;
        this.opacity = s.opacity;
        this.position = new Vector2f(s.position);
        this.size = new Vector2f(s.size);             
    }
    
    public Sprite(Image img,int offsetX,int offsetY,int sizeX,int sizeY) {
        this.image = img.getSubImage(offsetX, offsetY, sizeX, sizeY);
        this.size.x = sizeX;
        this.size.y = sizeY;
    }
    
    /**
     * Retourne l'opacité du sprite : 0 pour transparent, 1 pour opaque (sauf texture)
     * @return l'opacité du sprite
     */
    public float getOpacity() {
        return opacity;
    }

    /**
     * Applique l'opacité du sprite : 0 pour transparent, 1 pour opaque (sauf texture)
     * @param opacity opacité à appliquer
     */
    public void setOpacity(float opacity) {
        this.opacity = opacity;
    }


    /**
     * Retourne la position du sprite
     * @return Coordonnées 2d en pixels
     */
    public Vector2f getPosition() {
        return position;
    }

    /**
     * Applique la position du sprite
     * @param position Coordonnées 2d en pixels
     */
    public void setPosition(Vector2f position) {
        this.position = position;
    }

    /**
     * Retourne la taille du sprite (en pixels)
     * @return 
     */
    public Vector2f getSize() {
        return size;
    }

    /**
     * Applique la taille du sprite
     * @param size 
     */
    public void setSize(Vector2f size) {
        this.size = size;
    }

    /**
     * Retourne la distance par rapport à la vue <br/>
     * <ul>
     *      <li> sous ou egal à 0 - fixe, en premier plan</li>
     *      <li> au-dessus de 0 - dépendant de la vue, caché et cachant des objets </li>
     *      <li> 1 - normal </li>
     * </ul>  
     * @return 
     */
    public float getDepth() {
        return depth;
    }

    /**
     *
     * Applique la distance par rapport à la vue <br/>
     * <ul>
     *      <li> sous ou egal à 0 - fixe, en premier plan</li>
     *      <li> au-dessus de 0 - dépendant de la vue, caché et cachant des objets </li>
     *      <li> 1 - normal </li>
     * </ul>  
     * @param depth distance à appliquer
     */
    public void setDepth(float depth) {
        this.depth = depth;
    }
    
    /**
     * Retourne la couleur du sprite
     * @return 
     */
    public Color getColor() {
        return color;
    }
    
    /**
     * Applique la couleur du Sprite
     * @param color 
     */
    public void setColor(Color color) {
        this.color = color;
    }

    
    /**
     * Retourne true si la texture sera appliquée en miroir sur l'axe x
     * @return 
     */
    public boolean isFlipX() {
        return flipX;
    }

    /**
     * Indique si la texture sera appliquée en miroir sur l'axe x
     * @param flipX 
     */
    public void setFlipX(boolean flipX) {
        this.flipX = flipX;
    }

    /**
     * Retourne true si la texture sera appliquée en miroir sur l'axe y
     * @return 
     */
    public boolean isFlipY() {
        return flipY;
    }

    /**
     * Indique si la texture sera appliquée en miroir sur l'axe y
     * @param flipX 
     */
    public void setFlipY(boolean flipY) {
        this.flipY = flipY;
    }

    /**
     * Retourne la couche de rendu - 0 pour le premier plan, + pour l'arrière plan <br/>
     * @return 
     */
    public int getLayer() {
        return layer;
    }

    /**
     * Applique la couche de rendu - 0 pour le premier plan, + pour l'arrière plan <br/>
     * Ne pas utiliser, la classe Renderer s'occupe de ça avec Renderer.setSpriteLayer(Sprite sprite, int layer)
     * @param layer 
     */
    public void setLayer(int layer) {
        this.layer = layer;
    }
    
    
    public Image getImage() {
        return image;
    }
    
}
